/*
        UART to UDP demo for In-Circuit GW1000
        Radino32 UART demo for In-Circuit radino modules
        for more information: www.in-circuit.de or www.radino.cc

	Copyright (c) 2015 In-Circuit GmbH

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software
	and associated documentation files (the "Software"), to deal in the Software without restriction,
	including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
	LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
/*
        For USB-UART you need the STM32 Virtual COM Port Driver.
        http://www.st.com/content/st_com/en/products/development-tools/software-development-tools/stm32-software-development-tools/stm32-utilities/stsw-stm32102.html

        Write a text on UART to the GW1000 to send this text to your computer over LAN via UDP.
        To read this text on your computer you can use the Packet Sender (https://packetsender.com/).
*/


//---------------------------------------------------------
//----- INCLUDES ------------------------------------------
//---------------------------------------------------------

#include <SPI.h>
#if defined ARDUINO_ARCH_AVR
  #include <avr/wdt.h>
#elif defined ARDUINO_RADINO32
  #include <stm32/l1/iwdg.h>
#endif

// for UDP
#include <EtherCard_STM.h>

//---------------------------------------------------------
//----- CONSTANTS -----------------------------------------
//---------------------------------------------------------

#define outSerial       Serial   // Serial for USB, Serial1 for UART
#define SERIALSPEED     115200
#define BUFFERSIZE      100

//---------------------------------------------------------
//----- VARIABLES -----------------------------------------
//---------------------------------------------------------

// for UDP
static byte mymac[] = { 0xA1,0xB2,0xC3,0xD4,0xE5,0xF6 };    // MAC of GW1000
static byte destIp[] = { 192,168,178,10 };                  // IP of your computer
byte Ethernet::buffer[700];

const int dstPort PROGMEM = 50001;                          // UDP port of the coumputer
const int srcPort PROGMEM = 49999;                          // UDP port of GW1000

char sendText[BUFFERSIZE];
int messageStarttime = 0;
int MAXMessageWaittime = 50;

//---------------------------------------------------------
//----- MAIN ----------------------------------------------
//---------------------------------------------------------

void setup()
{
  // start serial communication for reading
  outSerial.begin(SERIALSPEED);

  // define start of setup
  pinMode(17,OUTPUT);
  digitalWrite(17, HIGH);
  
  // access ethernet controller
  while (ether.begin(sizeof Ethernet::buffer, mymac) == 0) 
  {
    outSerial.print("Failed to access Ethernet controller");
  }

  // request IP vis DHCP
  while (!ether.dhcpSetup())
  {
    outSerial.print("DHCP failed");
  }

  // show IP for debug
  outSerial.print("myIP: ");
  outSerial.print(ether.myip[0]);
  outSerial.print(".");
  outSerial.print(ether.myip[1]);
  outSerial.print(".");
  outSerial.print(ether.myip[2]);
  outSerial.print(".");
  outSerial.print(ether.myip[3]);
  
  // define end of setup  
  digitalWrite(17, LOW);
}

//---------------------------------------------------------

void loop()
{
  // send text via UDP if serial text found
  if (outSerial.available())
  {
    int count = 0;
    char tmpChar;
    messageStarttime = millis();
    
    do 
    {
      if (outSerial.available())
      {
        if (count < BUFFERSIZE)
        {
          tmpChar = outSerial.read();                         // read char from serial
          sendText[count] = tmpChar;                          // write char to array
          count++;
          sendText[count] = '\0';
        }
        if (count == BUFFERSIZE)                              // catch to long message
        {
          if (outSerial.available()){
            tmpChar = outSerial.read();                       // write unused chars to temp char
          }
        }
      }
    } while ((millis() - messageStarttime) < MAXMessageWaittime);  // time to collect all chars from serial
    ether.sendUdp(sendText, count, srcPort, destIp, dstPort );   // send text to computer via UDP
  }
}
